--[[
Populate filtered list

define window size
set starting position of window
display window

detect position in window
load items above or below window

]]





local matrix = {}

function matrix:New()
	local header = ledger.vars.header
	local header_reverse = ledger.vars.header_reverse
	local bg = ledger.vars.bg

	-- Instantiate
	local mat = iup.pdasubsubmatrix{numcol = 5, numcol_visible = 5, numlin = 0, numlin_visible = 25, hidefocus = 'YES', size='TWOTHIRD'}

	-- Initialize -- matrix headers, column alignments
	for _, v in pairs(header) do
		mat:setcell(0, v[1], v[2])
		mat["ALIGNMENT"..v[1]] = v[3]
	end

	function mat:setbg(lin, color)
		if (color) then self:setattribute("BGCOLOR", lin, -1, color) else self:setattribute("BGCOLOR", lin, -1, bg[lin % 2]) end
	end

	function mat:setfg(color, lin, col)
		if (col) then self['FGCOLOR'..lin..':'..col] = color else self:setattribute("FGCOLOR", lin, -1, bg[lin % 2]) end
	end

	function mat:setval(lin, col, val) 	self:setcell(lin, col, ' '..val..' ') end

	function mat:update()
		local list = ledger.list
		local count, total = 1, ledger.vars.startcredits
		local vfilter = ledger.filter.view
		local rlookup = ledger.filter.view.reverse_lookup
		local search = ledger.viewer.ui.searchbox.value:lower()

		local function checkfilter(major, minor, t)
			if (not rlookup[major]) then return false end
			if (not rlookup[major][minor]) then return false end
			if (not vfilter[major]) then return false end
			if (not vfilter[major].master) then return false end

			local master = vfilter[major].master
			local filt = vfilter[major]
			local idx = rlookup[major][minor]

			if (major == 'date') then 
				if (not t) then return nil end
				if (state_map[master.value]) then return filt.cmp:check(t) end
				return true
			end
			return state_map[master.value] or state_map[filt[idx].value]
		end

		local function setline(lin, tbl)
			for i = 1, #tbl do
				
				self:setval(lin, i, tbl[i])
				if (header_reverse[i] == 'amount' and tbl[i]:match('^%(')) then 
					self:setfg(lin, i, '255 50 50')
				else
					self:setfg(lin, i, '255 255 255')
				end
			end
		end

		local function escape_pattern(str)
			if (type(str) ~= 'string') then return nil, 'Expected string, got '..type(str)..'.' end
			return str:gsub('(%p)', '%%%1')
		end

		local function find_match(tbl, key)
			return tbl[header[key][1]]:lower():match(escape_pattern(search))
		end

		ledger.viewer.matrix_list_map = {}
		self.dellin = "1--1"
		-- Populate the matrix
		for i, v in ipairs(list) do
			local major, minor = v.category:match("(.-)%.(.*)")
			if (checkfilter(major, minor) and checkfilter('type', v.type:lower()) and checkfilter('date', 'check', v.time)) then
				self.numlin = count
				local lin_data = {}
				for k, w in pairs(v) do
					-- test.print('k='..tostring(k))
					if (k ~= 'category') then
						if (k == 'time') then 
							w = fuzzy_ledger.diff(os.time(), w, ledger.format.fuzziness)
							if (type(w) == 'number') then w = os.date(ledger.format.date, w) end
						elseif (k == 'amount') then
							total = total + w
							w = format_amount(w)
						end
						lin_data[header[k][1]] = w
					end
				end
				-- test.print(spickle(lin_data))
				-- Add the line to the matrix if searchbox is blank or the search pattern is found in reason or notes.
				if (search == '' or find_match(lin_data, 'reason') or find_match(lin_data, 'notes')) then 
					setline(count, lin_data)
					self:setbg(count, bg[count % 2])
					ledger.viewer.matrix_list_map[count] =  i
					count = count + 1
				end
 			end
		end
		ledger.viewer.ui.records.title = #ledger.viewer.matrix_list_map..' / '..#list
		ledger.viewer.ui.total.title = format_amount(total)
		if (ledger.viewer.ui.total.title:match('^%(')) then 
			ledger.viewer.ui.total.fgcolor = '255 50 50'
		else
			ledger.viewer.ui.total.fgcolor = '255 255 255'
		end
	end

	function mat:action(c, lin, col, active, after) 
		--test.print('c='..c..' / lin='..lin..' / col='..col..'/ active='..active..' / after='..after)
	end

	function mat:click_cb(lin, col)
		local sort_col = header_reverse[col]
		if (lin == 0) then
			sortlist(ledger.list, sort_col)
			self:update()
		else
			if (col ~= header.notes[1]) then return iup.IGNORE end
		end
	end

	function mat:enteritem_cb(lin, col)
		self:setbg(lin, bg[2])
		return iup.DEFAULT
	end

	function mat:leaveitem_cb(lin, col)
		self:setbg(lin, bg[lin % 2])
		return iup.DEFAULT
	end

	function mat:edition_cb(line, col, mode)
		-- test.print('line='..line..' / col='..col..' / mode='..mode)
		local list = ledger.list
		local map = ledger.viewer.matrix_list_map
		local edit = ledger.viewer.edit.ui
		if (mode == 1) then
			edit.dialog.curline = line
			edit.date:set(list[map[line]].date)
			edit.reason:set(list[map[line]].reason)
			edit.amount:set(format_amount(list[map[line]].amount))
			edit.textbox.value = list[map[line]].notes
			edit.dialog:popup(iup.CENTER, iup.CENTER)
			iup.Refresh(edit.dialog)
		end
		return iup.IGNORE
	end
	return mat
end























-- from if_pda_tab1.lua
local function create_player_matrix(columndefs, defsort, sortmodecb)
	local matrix
	local sortedlist
	local update_matrix
	local sort_key = defsort -- default to sort by distance
	local numcolumns = (#columndefs)
	local function set_sort_mode(mode)
		-- clicked on title of column
		sort_key = mode
		if sortmodecb then sortmodecb(mode) end
		-- color the text accordingly
		for i=1,numcolumns do
			matrix:setattribute("FGCOLOR", 0, i, mode == i and tabseltextcolor or tabunseltextcolor)
		end
	end

	matrix = iup.pdasubsubmatrix{
		numcol = numcolumns,
		expand = "YES",
		size="200x100",
	}
	for i=1,numcolumns do
		matrix["ALIGNMENT"..i] = columndefs[i].alignment
		matrix["0:"..i] = columndefs[i].title
	end
	set_sort_mode(defsort)
	function matrix:fgcolor_cb(row, col)
		local targetobj_nodeid = radar.GetRadarSelectionID()
		local colorindex = math.fmod(row,2)
		if targetobj_nodeid and targetobj_nodeid == GetPlayerNodeID(sortedlist[row]) then
			colorindex = 2
		end
		local c = bg_numbers[colorindex]
		return c[1],c[2],c[3],c[4],iup.DEFAULT
	end
	matrix.bgcolor_cb = matrix.fgcolor_cb
	function matrix:click_cb(row, col)
		if row == 0 then
			set_sort_mode(col)
			update_matrix(self)
		elseif columndefs.on_sel then
			columndefs.on_sel(sortedlist[row])
			update_matrix(self)
		end
	end
	function matrix:edition_cb()
		return iup.IGNORE
	end

	local function sort_list(sort_key, list)
		table.sort(list, columndefs[sort_key].fn)
	end

	local function reload_matrix(matrix, list)
		matrix.numlin = (#list)
	
		for index,v in ipairs(list) do
			columndefs.update_entry(matrix, index, v)
		end
	end
	
	local function reset_matrix(self, characterlist)
		matrix.dellin = "1--1"  -- one way of deleting all items in the matrix
		sortedlist = characterlist
		sort_list(sort_key, sortedlist)
		reload_matrix(matrix, sortedlist)
	end

	update_matrix = function(self)
		set_sort_mode(sort_key)
		sort_list(sort_key, sortedlist)
		reload_matrix(matrix, sortedlist)
	end

	matrix.update = update_matrix
	matrix.reset = reset_matrix

	return matrix
end


( plugins/ledger/ui_options.lua:134: bad argument #1 to 'ipairs' (table expected, got nil) )