
ledger.viewer.options            = {}
ledger.viewer.options.filter     = {}
ledger.viewer.options.ui         = {}
ledger.viewer.options.ui.dialog  = {}

local opt = ledger.viewer.options
local optui = opt.ui

local bool_map  = {[true] = 'ON', [false] = 'OFF'}
local state_map = {ON = true,  OFF = false}
local trans_map = {ON = 'YES', OFF = 'NO'}
local val_map   = {[true] = 1, [false] = 0}
local fuzz_map  = {
	{None = 'none', Seconds = 'minutes', Minutes = 'hours', Hours = 'days', Days = 'weeks', Weeks = 'months'},
	{none = 1, minutes = 2, hours = 3, days = 4, weeks = 5, months = 6},
	{'None', 'Seconds', 'Minutes', 'Hours', 'Days', 'Weeks'}
}

local fuzziness = ledger.format.fuzziness

local function xor(a, b) return ((a and not b) or (not a and b)) end
local function capitalize(str) return (str:gsub('^.', string.upper, 1)) end
local function fill(expand, size)
	local ctl = iup.fill{}
	if (expand) then ctl.expand = expand end
	if (size) then ctl.size = size end
	return ctl
end

local function toggle_state(tbl, property, state)
	if (type(tbl) ~= 'table' and type(tbl) ~= 'userdata') then return nil end

	if (type(tbl) == 'userdata') then
		if (property == 'value' and not iup.GetType(tbl) == 'toggle') then return end
		tbl[property] = state

	elseif (type(tbl) == 'table') then
		for _, v in pairs(tbl) do
			toggle_state(v, property, state)
		end
	end
	return true
end

local function toggle_active(tbl, yesno) 
	if (yesno ~= 'YES' and yesno ~= 'NO') then return nil end
	return toggle_state(tbl, 'active', yesno)
end

local function toggle_value(tbl, onoff)
	if (onoff ~= 'ON' and onoff ~= 'OFF') then return nil end
	return toggle_state(tbl, 'value', onoff)
end

local function toggle_visible(tbl, yesno) 
	if (yesno ~= 'YES' and yesno ~= 'NO') then return nil end
	return toggle_state(tbl, 'visible', yesno)
end

-- -----------------------------------------------------------

function opt.filter:New(listname)
	local rfilter = ledger.filter.record[listname]
	rfilter.controls = {}
	local controls = rfilter.controls
	local capname = capitalize(listname)
	local vbox = {}
	local subfilters = iup.vbox{}

	local function create_toggle(title, value, func)
		if (not title) then return nil, "Expected a string for title." end
		value = value or 'ON'
		func = func or function(self)
			local master = controls.master
			local onoff = 'ON'
			for _, v in ipairs(controls) do
				if (not state_map[v.value]) then onoff = 'OFF'; break end
			end
			if (xor(state_map[master.value], state_map[onoff])) then toggle_value(master, onoff) end
		end

		return iup.stationtoggle{title = title, value = value, action = func}
	end

	controls.master = create_toggle(capname, nil, function(self)
		local state = self.value
		toggle_value(controls, state)
	end)

	local count = 0
	for k, v in pairs(rfilter) do
		if (k ~= 'controls') then
			local tog = create_toggle(k, bool_map[v])
			count = count + 1
			controls[count] = tog
			iup.Append(subfilters, tog)
		end
	end
	controls[1]:action()

	vbox = iup.vbox{
		iup.hbox{controls.master, fill()},
		iup.hbox{fill(nil, 10),	subfilters,},
		fill(nil, 10),
	}

	return vbox
end


function opt.build_dialog()

	optui.credits = iup.text{value = ledger.vars.startcredits, size = 200}
	optui.dateformat = iup.text{value = ledger.format.date, size = 200}
	optui.fuzziness = iup.stationsubsublist{dropdown = 'YES', size = 150, action = function(self)
			fuzziness = fuzz_map[1][self[self.value]]
		end}
	for i, v in ipairs(fuzz_map[3]) do optui.fuzziness[i] = v end
	optui.fuzziness.value = fuzz_map[2][fuzziness]

	optui.save = iup.stationbutton{title = 'Save', action = function(self)
		local rfilter = ledger.filter.record
		local wInt = gkini.WriteInt
		local wStr = gkini.WriteString

		-- Save start credits
		ledger.vars.startcredits = optui.credits.value
		wInt('Ledger', 'startcredits', ledger.vars.startcredits)
		
		-- Save record filters
		for k, v in pairs(rfilter) do
			local controls = v.controls
			for _, ctl in ipairs(controls) do
				local key = ctl.title
				v[key] = (ctl.value == 'ON')
				wInt('Ledger', 'filter.record.'..k..'.'..key, val_map[v[key]])
			end
		end

		-- Save date format
		ledger.format.date = optui.dateformat.value
		wStr('Ledger', 'format.date', ledger.format.date)

		-- Save fuzziness level
		ledger.format.fuzziness = fuzziness
		wStr('Ledger', 'format.fuzziness', fuzziness)

		optui.cancel:action()
	end}

	optui.cancel = iup.stationbutton{title = 'Cancel', action = function() 
		optui.dialog:hide()
	end}

	local credits = iup.hbox{
		iup.label{title = 'Starting credits'},
		optui.credits,
		gap = 5,
	}

	local filters = iup.vbox{
		iup.label{title = 'Record Filters'},
		opt.filter:New('guild'),
		opt.filter:New('mission'),
		opt.filter:New('player'),
		opt.filter:New('station'),
		opt.filter:New('ship'),
		gap = 5,
	}

	local dateformat = iup.vbox{
		iup.hbox{iup.label{title = 'Date Format'}, optui.dateformat, gap = 5},
		iup.hbox{iup.label{title = 'Time Fuzziness'}, optui.fuzziness, gap = 5},
		gap = 5,
	}

	optui.main = iup.pdarootframe{
		iup.pdarootframebg{
			iup.vbox{
				credits,
				filters,
				dateformat,
				iup.hbox{
					iup.fill{},
					optui.save,
					optui.cancel,
					gap = 10,
				},
				gap=10,
			},
		},
		expand="NO",
	}

	optui.dialog = iup.dialog{
		iup.vbox{
			iup.fill{},
				iup.hbox{
					iup.fill{},
					optui.main,
					iup.fill{},
				},
			iup.fill{},
		},
		defaultesc = optui.close,
		--bgcolor = "0 0 0 128 *",
		border = "NO",
		resize = "NO",
		menubox = "NO",
		topmost = "YES",

	}

	optui.dialog:map()
end
